#include "ptask.h"
#include "pmutex.h"
#include <semaphore.h>
#include <stdio.h>

// Producer/consumer demonstration code
// based on aperiodic tasks with deadlines.
// The produces produces twice as fast as the consumer consumes
// and it is expected that the producer will start missing
// deadlines after MAXBUFF * TIMETOCONSUME  producer ticks.
// Peter Walsh csci 461

#define TIMETOPRODUCE 1
#define TIMETOCONSUME 2
#define MAXBUFF 5

// define mutex mx
pthread_mutex_t mx; 

// define semaphores full and empty
sem_t full; 
sem_t empty;
 
// example usage 
//    pthread_mutex_lock(&mx);
//    pthread_mutex_unlock(&mxa);
//    sem_wait(&full);
//    sem_post(&full);


ptask producer() {

    int t;
    int tick;

    tick = 0;
    ptask_wait_for_activation();

    while (1) {

        printf("Producer Tick %d\n", tick);
        tick = tick + 1;

        if (ptask_deadline_miss()) {
           printf("Producer  missed deadline \n");
        }

        t = ptask_get_index();
        printf("Producer Task T%d is executing \n", t);
        sem_wait(&empty);
        printf("Producer Task T%d has decreased the number of empty locations  \n", t);
        pthread_mutex_lock(&mx);
        printf("Producer Task T%d has exclusive access to buffer \n", t);
        printf("Producer Task T%d has PRODUCED \n", t);
        pthread_mutex_unlock(&mx);
        printf("Producer Task T%d has unlocked access to buffer \n", t);
        sem_post(&full);
        printf("Producer Task T%d  has increased the number of full locations \n", t);
        printf("Producer Task T%d  is suspending (for next period) \n", t);

        ptask_wait_for_period();
    }
}

ptask consumer() {

    int t;
    int tick;

    tick = 0;
    ptask_wait_for_activation();

    while (1) {

        printf("Consumer Tick %d\n", tick);
        tick = tick + 1;

        if (ptask_deadline_miss()) {
           printf("Consumer missed deadline");
        }

        t = ptask_get_index();
        printf("Consumer Task T%d is executing \n", t);
        sem_wait(&full);
        printf("Consumer Task T%d has decreased the number of full locations  \n", t);
        pthread_mutex_lock(&mx);
        printf("Consumer Task T%d has exclusive access to buffer \n", t);
        printf("Consumer Task T%d has CONSUMED \n", t);
        pthread_mutex_unlock(&mx);
        printf("Consumer Task T%d has unlocked access to buffer \n", t);
        sem_post(&empty);
        printf("Consumer Task T%d  has increased the number of empty locations \n", t);
        printf("Producer Task T%d  is suspending (for next period) \n", t);

        ptask_wait_for_period();
    }
}

int main() {
    tpars param_p1, param_c1;
    int p1, c1;
    int num;

    // Initialize full and empty.
    sem_init(&full, 0, 0);
    sem_init(&empty, 0, MAXBUFF);

    // Initialize mx and modify attributes to
    // support priority inheritance protocol.
    pthread_mutex_init(&mx, NULL);
    pmux_create_pi(&mx);

    ptask_init(SCHED_FIFO, PARTITIONED, PRIO_INHERITANCE);

    ptask_param_init(param_p1);              
    ptask_param_period(param_p1, TIMETOPRODUCE, SEC);
    ptask_param_deadline(param_p1, TIMETOPRODUCE, SEC); 
    ptask_param_priority(param_p1, 4);   
    ptask_param_processor(param_p1, 0);     
    p1 = ptask_create_param(producer, &param_p1);
    printf("Creation of task: %d\n", p1);

    ptask_param_init(param_c1);              
    ptask_param_period(param_c1, TIMETOCONSUME, SEC);
    ptask_param_deadline(param_c1, TIMETOCONSUME, SEC); 
    ptask_param_priority(param_c1, 6);   
    ptask_param_processor(param_c1, 0);     
    c1 = ptask_create_param(consumer, &param_c1);
    printf("Creation of task: %d\n", c1);

    num = ptask_getnumcores();
    printf("Num cores: %d\n", num);

    ptask_activate(p1);
    printf("Activation of task: %d\n", p1);

    ptask_activate(c1);
    printf("Activation of task: %d\n", c1);

    while (1) {
       usleep(100000); 
    }

    return 0;
}

